@@ -13,6 +13,7 @@ from lilya.conf.exceptions import FieldException
 from lilya.conf.global_settings import Settings
 from lilya.datastructures import State, URLPath
 from lilya.dependencies import wrap_dependency
+from lilya.lifecycle import get_hooks as _lifecycle_get_hooks
 from lilya.logging import LoggingConfig, setup_logging
 from lilya.middleware.asyncexit import AsyncExitStackMiddleware
 from lilya.middleware.base import DefineMiddleware
@@ -1034,6 +1035,10 @@ class Lilya(RoutingMethodsMixin, BaseLilya):
         _dependencies = self.load_settings_value("dependencies", dependencies) or {}
         self.dependencies = {key: wrap_dependency(dep) for key, dep in _dependencies.items()}
 
+        # Used for testing purposes
+        # Allows overriding dependencies in tests
+        self.dependency_overrides: dict[str, Any] = {}
+
         self.logging_config = self.load_settings_value("logging_config", logging_config)
         self.serializer_config = self.load_settings_value("serializer_config", serializer_config)
         self.state = State()
@@ -1053,6 +1058,19 @@ class Lilya(RoutingMethodsMixin, BaseLilya):
             "include_in_schema", include_in_schema, is_boolean=True
         )
 
+        if lifespan is None:
+            # Merge global lifecycle hooks into app-level hooks
+            hooks = _lifecycle_get_hooks()
+            if on_startup:
+                on_startup = [*hooks["startup"], *on_startup]
+            else:
+                on_startup = hooks["startup"]
+
+            if on_shutdown:
+                on_shutdown = [*on_shutdown, *hooks["shutdown"]]
+            else:
+                on_shutdown = hooks["shutdown"]
+
         if self.router_class is not None:
             self.router = self.router_class(
                 routes=routes,
@@ -1079,6 +1097,19 @@ class Lilya(RoutingMethodsMixin, BaseLilya):
         if self.register_as_global_instance:
             _monkay.set_instance(self)
 
+    def override_dependency(self, name: str, override: Callable[..., Any]) -> None:
+        """
+        Register a global override for a dependency by name.
+        The override will be applied anywhere the dependency is used.
+        """
+        self.dependency_overrides[name] = wrap_dependency(override)
+        self.router.dependency_overrides.update(self.dependency_overrides)
+
+    def reset_dependency_overrides(self) -> None:
+        """Clear all registered dependency overrides."""
+        self.dependency_overrides.clear()
+        self.router.dependency_overrides.clear()
+
     def configure_openapi(self, openapi_config: Any | None = None) -> None:
         from lilya.contrib.openapi.config import OpenAPIConfig
 
