@@ -41,7 +41,7 @@ def safe_get_type_hints(cls: type) -> dict[str, Any]:
     """
     try:
         return get_type_hints(cls, include_extras=True)
-    except Exception:
+    except Exception:  # noqa
         return cls.__annotations__
 
 
@@ -124,7 +124,7 @@ class BaseSettings:
             if typ is bool or str(typ) == "bool":
                 return value.lower() in self.__truthy__
             return typ(value)
-        except Exception:
+        except Exception:  # noqa
             if get_origin(typ) is Union or get_origin(UnionType):
                 type_name = " | ".join(
                     t.__name__ if hasattr(t, "__name__") else str(t) for t in get_args(typ)
@@ -171,7 +171,7 @@ class BaseSettings:
                         continue
                     result_key = name.upper() if upper else name
                     result[result_key] = value
-                except Exception:
+                except Exception:  # noqa
                     # Skip properties that raise errors
                     continue
 
@@ -246,6 +246,48 @@ class Internal(ContribSettings):
         ),
     ] = "~/.config/ptpython/config.py"
 
+    @property
+    def runserver_theme(self) -> dict[str, str]:
+        """
+        Returns a dictionary defining the color theme for the terminal UI
+        when running the runserver.
+
+        The theme includes styles for various elements such as tags, placeholders,
+        text, selected items, results, progress indicators, errors, and log info.
+
+        **Example**
+
+        ```python
+        from lilya.conf import Settings
+
+        class AppSettings(Settings):
+            @property
+            def runserver_theme(self) -> dict[str, str]:
+                return {
+                    "tag.title": "white on #02a6f2",
+                    "tag": "white on #025af2",
+                    "placeholder": "grey85",
+                    "text": "white",
+                    "selected": "#025af2",
+                    "result": "grey85",
+                    "progress": "on #025af2",
+                    "error": "red",
+                    "log.info": "black on blue",
+                }
+        ```
+        """
+        return {
+            "tag.title": "white on #02a6f2",
+            "tag": "white on #025af2",
+            "placeholder": "grey85",
+            "text": "white",
+            "selected": "#025af2",
+            "result": "grey85",
+            "progress": "on #025af2",
+            "error": "red",
+            "log.info": "black on blue",
+        }
+
 
 class Settings(Internal):
     debug: Annotated[
@@ -496,6 +538,14 @@ class Settings(Internal):
             """
         ),
     ] = True
+    csrf_token_name: Annotated[
+        str,
+        Doc(
+            """
+            Name given to the `csrf_token` field for the HTML form.
+            """
+        ),
+    ] = "csrf_token"
 
     @property
     def routes(self) -> list[Any]:
