@@ -6,8 +6,9 @@ from functools import cached_property, lru_cache, wraps
 from types import GeneratorType
 from typing import Any, cast
 
+from lilya._internal._scopes import scope_manager
 from lilya.compat import is_async_callable, run_sync
-from lilya.enums import SignatureDefault
+from lilya.enums import Scope, SignatureDefault
 from lilya.requests import Request
 from lilya.websockets import WebSocket
 
@@ -50,12 +51,23 @@ class Provide:
         dependency: Callable[..., Any],
         *args: Any,
         use_cache: bool = False,
+        scope: Scope | str | None = Scope.REQUEST,
         **kwargs: Any,
     ) -> None:
         self.dependency = dependency
         self.provided_args = args
         self.provided_kwargs = kwargs
         self.use_cache = use_cache
+
+        if isinstance(scope, str):
+            try:
+                scope = Scope[scope.upper()]
+            except KeyError as exc:
+                raise ValueError(
+                    f"Invalid scope '{scope}'. Use one of: {[s.name for s in Scope]}"
+                ) from exc
+
+        self.scope = scope
         self._cache: Any = None
         self._resolved: bool = False
         self.__dependency_signature__: inspect.Signature | None = None
@@ -79,6 +91,57 @@ class Provide:
         if self.use_cache and self._resolved:
             return self._cache
 
+        if self.scope in (Scope.APP, Scope.GLOBAL):
+
+            async def _factory() -> Any:
+                # Delegate to internal resolution logic
+                return await self._resolve_internal(request, dependencies_map)
+
+            if self.scope == Scope.APP:
+                app = getattr(request, "app", None)
+                attr_name = f"__lilya_app_depkey_{id(self.dependency)}"
+
+                if app is not None:
+                    # real Lilya app — reuse per-app wrapper
+                    dep_key = getattr(app, attr_name, None)
+                    if dep_key is None:
+
+                        def _app_scoped_key(*args: Any, **kwargs: Any) -> Any:
+                            return self.dependency(*args, **kwargs)
+
+                        setattr(app, attr_name, _app_scoped_key)
+                        dep_key = _app_scoped_key
+                else:
+                    # fallback for no app, stable per Provide key
+                    if not hasattr(self, "_app_scope_key"):
+
+                        def _app_scoped_key(*args: Any, **kwargs: Any) -> Any:
+                            return self.dependency(*args, **kwargs)
+
+                        self._app_scope_key = _app_scoped_key
+                    dep_key = self._app_scope_key
+            else:
+                dep_key = self.dependency
+
+            # retrieve or create from scope_manager
+            return await scope_manager.get_or_create(
+                self.scope,
+                dep_key,  # callable identity
+                _factory,
+            )
+
+        return await self._resolve_internal(request, dependencies_map)
+
+    async def _resolve_internal(
+        self,
+        request: Request,
+        dependencies_map: dict[str, Self | Any],
+    ) -> Any:
+        """
+        Internal dependency resolution logic.
+        Used by resolve() and scope_manager to separate
+        lifetime control from the actual call logic.
+        """
         # If the dependency is a class, we need to instantiate it
         # and pass the request to it
         if self.__signature__.parameters:
@@ -88,7 +151,7 @@ class Provide:
             if isinstance(request, Request):
                 try:
                     json_data = cast(dict[str, Any], await request.data()) or {}
-                except Exception:
+                except Exception:  # noqa
                     ...
 
                 updated_keys: dict[str, Any] = {}
@@ -354,7 +417,7 @@ class PureScope:
                     await fn()
                 else:
                     fn()
-            except Exception:
+            except Exception:  # noqa
                 ...
 
     async def __aenter__(self) -> "PureScope":
@@ -404,6 +467,7 @@ class _Depends:
         dependency: Callable[..., Any] | Any,
         *args: Any,
         use_cache: bool = False,
+        scope: Scope | str | None = Scope.REQUEST,
         **kwargs: Any,
     ) -> None:
         if not isinstance(dependency, _Depends) and not callable(dependency):
@@ -412,6 +476,16 @@ class _Depends:
         self.provided_args = args
         self.provided_kwargs = kwargs
         self.use_cache = use_cache
+
+        if isinstance(scope, str):
+            try:
+                scope = Scope[scope.upper()]
+            except KeyError as exc:
+                raise ValueError(
+                    f"Invalid scope '{scope}'. Use one of: {[s.name for s in Scope]}"
+                ) from exc
+
+        self.scope = scope
         self._cache: Any = None
         self._resolved: bool = False
         self.__dependency_signature__: inspect.Signature | None = None
@@ -444,6 +518,34 @@ class _Depends:
         dependencies_map = dependencies_map or {}
         overrides = overrides or {}
 
+        if self.scope in (Scope.APP, Scope.GLOBAL):
+
+            async def _factory() -> Any:
+                # the actual internal resolution logic below
+                return await self._resolve_internal(dependencies_map, overrides, scope)
+
+            if isinstance(self.dependency, _Depends):
+
+                def dep_callable() -> Any:
+                    return self.dependency
+            elif callable(self.dependency):
+                dep_callable = self.dependency
+            else:
+                dep_callable = _constant(self.dependency)
+
+            return await scope_manager.get_or_create(
+                self.scope,
+                dep_callable,
+                _factory,
+            )
+        return await self._resolve_internal(dependencies_map, overrides, scope)
+
+    async def _resolve_internal(
+        self,
+        dependencies_map: dict[str, Any],
+        overrides: dict[Any, Any],
+        scope: PureScope | None,
+    ) -> Any:
         # If args/kwargs explicitly provided or this is a class, call directly.
         if self.provided_args or self.provided_kwargs or inspect.isclass(self.dependency):
             dep_callable = overrides.get(self.dependency, self.dependency)
