@@ -3,6 +3,7 @@ from __future__ import annotations
 import contextlib
 import functools
 import http.cookies
+import inspect
 import os
 import stat
 import time
@@ -12,6 +13,7 @@ from collections.abc import (
     AsyncIterable,
     Awaitable,
     Callable,
+    Coroutine,
     Generator,
     Iterable,
     Mapping,
@@ -40,8 +42,20 @@ from lilya.concurrency import iterate_in_threadpool
 from lilya.datastructures import URL, Header
 from lilya.encoders import ENCODER_TYPES, EncoderProtocol, MoldingProtocol, json_encode
 from lilya.enums import Event, HTTPMethod, MediaType
+from lilya.logging import logger
 from lilya.ranges import ContentRanges, Range, parse_range_value
-from lilya.types import Receive, Scope, Send
+from lilya.serializers import serializer
+from lilya.types import Message, Receive, Scope, Send
+
+try:
+    import yaml
+except ImportError:  # pragma: no cover
+    yaml = None
+
+try:
+    import msgpack
+except ImportError:  # pragma: no cover
+    msgpack = None
 
 Content = str | bytes
 Encoder = EncoderProtocol | MoldingProtocol
@@ -309,6 +323,12 @@ class HTMLResponse(Response):
     media_type = MediaType.HTML
 
 
+class HTML(HTMLResponse):
+    """
+    A simple alias for HTMLResponse for better readability.
+    """
+
+
 class Error(HTMLResponse):
     status_code = status.HTTP_500_INTERNAL_SERVER_ERROR
 
@@ -317,6 +337,9 @@ class PlainText(Response):
     media_type = MediaType.TEXT
 
 
+class TextResponse(PlainText): ...
+
+
 class JSONResponse(Response):
     media_type = MediaType.JSON
 
@@ -453,6 +476,293 @@ class StreamingResponse(Response):
             await self.background()
 
 
+class EventStreamResponse(Response):
+    """
+    Server-Sent Events (SSE) streaming response for Lilya.
+
+    Fully async and anyio-native, matching Starlette's EventSourceResponse
+    behavior while keeping Lilya conventions.
+
+    Supports:
+    - async and sync iterables
+    - pings (keep-alive comments)
+    - send timeout
+    - graceful shutdown
+    - client disconnect detection
+    """
+
+    DEFAULT_PING_INTERVAL = 15
+    DEFAULT_SEPARATOR = "\n"
+
+    media_type = "text/event-stream"
+
+    def __init__(
+        self,
+        content: AsyncIterable[dict[str, Any]],
+        *,
+        status_code: int = 200,
+        headers: Mapping[str, str] | None = None,
+        media_type: str = "text/event-stream",
+        background: Task | None = None,
+        retry: int | None = None,
+        ping_interval: int | None = None,
+        separator: str | None = None,
+        ping_message_factory: Callable[[], dict[str, Any]] | None = None,
+        send_timeout: float | None = None,
+        data_sender_callable: Callable[[], Coroutine[None, None, None]] | None = None,
+        client_close_handler: Callable[[Message], Awaitable[None]] | None = None,
+    ) -> None:
+        if separator not in (None, "\r\n", "\r", "\n"):
+            raise ValueError(f"sep must be one of: \\r\\n, \\r, \\n, got: {separator}")
+        self.sep = separator or self.DEFAULT_SEPARATOR
+        self.retry = retry
+
+        if inspect.isasyncgen(content) or isinstance(content, AsyncIterable):
+
+            async def async_bytes_gen() -> AsyncIterable[bytes]:
+                async for event in content:
+                    yield self._encode_event(event)
+
+            self.body_iterator = async_bytes_gen()
+
+        else:
+
+            async def sync_bytes_gen() -> AsyncIterable[bytes]:  # type: ignore
+                for event in content:
+                    yield self._encode_event(event)
+
+            self.body_iterator = sync_bytes_gen()
+
+        self.status_code = status_code
+        self.media_type = media_type or self.media_type
+        self.background = background
+        self.ping_message_factory = ping_message_factory
+
+        ping_interval = ping_interval or self.DEFAULT_PING_INTERVAL
+        send_timeout = send_timeout
+
+        self.ping_interval = (
+            ping_interval * 1000 if ping_interval is not None else self.DEFAULT_PING_INTERVAL
+        )
+        self.send_timeout = send_timeout * 1000 if send_timeout is not None else None
+        self.timeout_in_seconds = send_timeout if send_timeout else None
+
+        self.data_sender_callable = data_sender_callable
+        self.client_close_handler = client_close_handler
+        self.active = True
+        self._send_lock = anyio.Lock()
+
+        default_headers = {
+            "Cache-Control": "no-store",
+            "Connection": "keep-alive",
+            "X-Accel-Buffering": "no",
+            "Transfer-Encoding": "chunked",
+        }
+        if headers:
+            default_headers.update(headers)
+
+        default_headers.pop("Content-Length", None)
+
+        super().__init__(
+            content=None,
+            status_code=status_code,
+            headers=default_headers,
+            media_type=self.media_type,
+            background=background,
+        )
+
+    async def _send_chunk(self, send: Send, data: str) -> None:
+        if not data.endswith("\n\n"):
+            data += "\n\n"
+        try:
+            with anyio.fail_after(self.send_timeout):
+                await send(
+                    {
+                        "type": "http.response.body",
+                        "body": data.encode("utf-8"),
+                        "more_body": True,
+                    }
+                )
+        except TimeoutError as exc:
+            raise TimeoutError("SSE send timed out") from exc
+
+    async def _stream_response(self, send: Send) -> None:
+        """
+        Stream Server-Sent Events (SSE) to the client.
+        Includes per-event timeout and graceful closure.
+        """
+        await send(
+            {
+                "type": "http.response.start",
+                "status": self.status_code,
+                "headers": self.raw_headers,
+            }
+        )
+
+        aiter_obj = self.body_iterator.__aiter__()
+
+        try:
+            while True:
+                try:
+                    if self.send_timeout is not None:
+                        try:
+                            # Wait for the next event with a timeout.
+                            with anyio.fail_after(self.send_timeout):
+                                event = await aiter_obj.__anext__()
+                        except TimeoutError:
+                            logger.warning(
+                                "SSE send timed out after %.3fs", self.timeout_in_seconds
+                            )
+                            self.active = False
+                            async with self._send_lock:
+                                await send(
+                                    {"type": "http.response.body", "body": b"", "more_body": False}
+                                )
+                            raise TimeoutError("SSE send timed out") from None
+                        except anyio.get_cancelled_exc_class():
+                            logger.warning(
+                                "SSE send timed out after %.3fs", self.timeout_in_seconds
+                            )
+                            self.active = False
+                            async with self._send_lock:
+                                await send(
+                                    {"type": "http.response.body", "body": b"", "more_body": False}
+                                )
+                            raise TimeoutError("SSE send timed out") from None
+                    else:
+                        event = await aiter_obj.__anext__()
+
+                except StopAsyncIteration:
+                    break
+
+                # Encode & send normally
+                chunk = self._encode_event(event)
+                await self._send_chunk(send, chunk.decode() if isinstance(chunk, bytes) else chunk)
+
+        finally:
+            async with self._send_lock:
+                self.active = False
+                await send({"type": "http.response.body", "body": b"", "more_body": False})
+
+    async def _ping(self, send: Send) -> None:
+        """
+        Periodically sends ping comments or custom ping messages.
+        """
+        if not self.ping_interval:
+            return
+
+        try:
+            # Immediate first ping
+            ping_event = (
+                self.ping_message_factory() if self.ping_message_factory else {":": "ping"}
+            )
+            await send(
+                {
+                    "type": "http.response.body",
+                    "body": self._encode_event(ping_event),
+                    "more_body": True,
+                }
+            )
+
+            # Give control back to event loop to allow _stream_response to close
+            await anyio.sleep(0)
+
+            # Exit early if stream already done (short-lived)
+            if not self.active:
+                return
+
+            # Continue periodic pings
+            while self.active:
+                await anyio.sleep(self.ping_interval)
+                ping_event = (
+                    self.ping_message_factory() if self.ping_message_factory else {":": "ping"}
+                )
+                await send(
+                    {
+                        "type": "http.response.body",
+                        "body": self._encode_event(ping_event),
+                        "more_body": True,
+                    }
+                )
+        except Exception:
+            ...
+
+    async def _listen_for_disconnect(self, receive: Receive) -> None:
+        """
+        Watch for a disconnect message from the client.
+        """
+        while self.active:
+            message = await receive()
+            if message["type"] == "http.disconnect":
+                self.active = False
+                logger.debug("Client disconnected from SSE stream.")
+                if self.client_close_handler:
+                    await self.client_close_handler(message)
+                break
+
+    async def _listen_for_exit_signal(self) -> None:
+        """
+        Hook for future Lilya shutdown signals (if available).
+        Currently just a placeholder for Starlette-like behavior.
+        """
+        # could integrate Lilya AppStatus here if available
+        await anyio.sleep_forever()
+
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
+        """
+        Entrypoint for Lilya's ASGI contract.
+        """
+        async with anyio.create_task_group() as tg:
+
+            async def cancel_on_finish() -> None:
+                await self._stream_response(send)
+                if self.active:
+                    tg.cancel_scope.cancel()
+
+            async def cancel_on_disconnect() -> None:
+                await self._listen_for_disconnect(receive)
+                tg.cancel_scope.cancel()
+
+            tg.start_soon(cancel_on_finish)
+            tg.start_soon(self._ping, send)
+            tg.start_soon(cancel_on_disconnect)
+            tg.start_soon(self._listen_for_exit_signal)
+
+            if self.data_sender_callable:
+                tg.start_soon(self.data_sender_callable)
+
+        if self.background:
+            await self.background()
+
+    def _encode_event(self, event: dict[str, Any] | bytes) -> bytes:
+        """
+        Encodes a single event dictionary into SSE-compliant text bytes.
+        """
+        if isinstance(event, bytes):
+            return event
+
+        lines: list[str] = []
+
+        if ":" in event:  # comment line
+            lines.append(f": {event[':']}")
+
+        if "id" in event:
+            lines.append(f"id: {event['id']}")
+        if "event" in event:
+            lines.append(f"event: {event['event']}")
+        if "data" in event:
+            data = event["data"]
+            if isinstance(data, (dict, list)):
+                data = serializer.dumps(data, separators=(", ", ": "))
+            lines.append(f"data: {data}")
+        if "retry" in event or self.retry:
+            retry = event.get("retry", self.retry)
+            if retry is not None:
+                lines.append(f"retry: {retry}")
+
+        return (self.sep.join(lines) + self.sep * 2).encode("utf-8")
+
+
 class FileResponse(Response):
     chunk_size = 64 * 1024
 
@@ -762,6 +1072,129 @@ class TemplateResponse(HTMLResponse):
         await super().__call__(scope, receive, send)
 
 
+class CSVResponse(Response):
+    media_type = "text/csv"
+
+    def make_response(self, content: Iterable[dict[str, Any]] | None) -> bytes:
+        """
+        Converts a list of dictionaries to a CSV formatted byte string.
+
+        Args:
+            content (Iterable[dict[str, Any]] | None): Iterable of dicts representing the CSV rows
+        """
+        if not content:
+            return b""
+
+        rows = list(content)
+        if not rows:
+            return b""
+
+        headers = list(rows[0].keys())
+        lines = [",".join(headers)]
+        for row in rows:
+            lines.append(",".join(str(row.get(h, "")) for h in headers))
+        return "\n".join(lines).encode()
+
+
+class XMLResponse(Response):
+    media_type = "application/xml"
+
+    def make_response(self, content: Any) -> bytes:
+        """
+        Converts a Python object (dict, list, str, bytes) to an XML formatted byte string.
+
+        Args:
+            content (Any): The content to be converted to XML.
+        """
+        if content is None:
+            return b""
+
+        if isinstance(content, (bytes, bytearray)):
+            return bytes(content)
+
+        if isinstance(content, str):
+            return content.encode(self.charset)
+
+        def to_xml(value: Any, tag: str) -> Any:
+            # For dict: recurse into children
+            if isinstance(value, dict):
+                inner = "".join(to_xml(v, k) for k, v in value.items())
+                return f"<{tag}>{inner}</{tag}>"
+
+            # For list: repeat same tag for each element
+            elif isinstance(value, list):
+                return "".join(to_xml(item, tag) for item in value)
+            else:
+                return f"<{tag}>{value}</{tag}>"
+
+        # When top-level is a dict, wrap each key directly
+        if isinstance(content, dict):
+            xml_str = "".join(to_xml(v, k) for k, v in content.items())
+            xml_str = f"<root>{xml_str}</root>"
+
+        # When top-level is a list, wrap each item in <root>
+        elif isinstance(content, list):
+            xml_str = "".join(to_xml(item, "root") for item in content)
+        else:
+            xml_str = f"<root>{content}</root>"
+
+        return xml_str.encode(self.charset)
+
+
+class YAMLResponse(Response):
+    media_type = "application/x-yaml"
+
+    def make_response(self, content: Any) -> bytes:
+        """
+        Converts a Python object to a YAML formatted byte string.
+
+        Args:
+            content (Any): The content to be converted to YAML.
+        """
+        if content is None:
+            return b""
+        return cast(bytes, yaml.safe_dump(content, sort_keys=False).encode(self.charset))
+
+
+class MessagePackResponse(Response):
+    media_type = "application/x-msgpack"
+
+    def make_response(self, content: Any) -> bytes:
+        """
+        Converts a Python object to a MessagePack formatted byte string.
+        """
+        if content is None:
+            return b""
+        return cast(bytes, msgpack.packb(content, use_bin_type=True))
+
+
+class NDJSONResponse(Response):
+    media_type = "application/x-ndjson"
+
+    def make_response(self, content: Iterable[dict[str, Any]] | None) -> bytes:
+        """
+        Converts an iterable of dictionaries to a NDJSON formatted byte string.
+        """
+        if not content:
+            return b""
+
+        lines = [serializer.dumps(item, separators=(",", ": ")) for item in content]
+        return ("\n".join(lines)).encode(self.charset)
+
+
+class ImageResponse(Response):
+    def __init__(
+        self,
+        content: bytes,
+        media_type: str = "image/png",
+        status_code: int = 200,
+        headers: dict[str, str] | None = None,
+    ):
+        super().__init__(
+            content=content, status_code=status_code, headers=headers, media_type=media_type
+        )
+
+
 def make_response(
     content: Any,
     response_class: type[Response] = JSONResponse,
@@ -769,13 +1202,15 @@ def make_response(
     headers: Mapping[str, str] | None = None,
     background: Task | None = None,
     encoders: Sequence[Encoder | type[Encoder]] | None = None,
-    # passing mutables as default argument is not a good style but here is no other way
-    json_encode_extra_kwargs: dict | None = {},  # noqa: B006
+    json_encode_extra_kwargs: dict | None = None,  # noqa: B006
 ) -> Response:
     """
     Build JSON responses from a given content and
     providing extra parameters.
     """
+    if json_encode_extra_kwargs is None:
+        json_encode_extra_kwargs = {}
+
     with response_class.with_transform_kwargs(json_encode_extra_kwargs):
         return response_class(
             content=content,
@@ -785,32 +1220,3 @@ def make_response(
             background=background,
             encoders=encoders,
         )
-
-
-def redirect(
-    url: str | URL,
-    status_code: int = status.HTTP_307_TEMPORARY_REDIRECT,
-    headers: Mapping[str, str] | None = None,
-    background: Task | None = None,
-    encoders: Sequence[Encoder | type[Encoder]] | None = None,
-) -> RedirectResponse:
-    """
-    Redirect to a different URL.
-
-    Args:
-        url (Union[str, URL]): The URL to redirect to.
-        status_code (int, optional): The status code of the redirect response (default is 307).
-        headers (Union[Mapping[str, str], None], optional): Additional headers to include (default is None).
-        background (Union[Task, None], optional): A background task to run (default is None).
-        encoders (Union[Sequence[Encoder | type[Encoder]], None], optional): A sequence of encoders to use (default is None).
-
-    Returns:
-        RedirectResponse: A response object that redirects to the given URL.
-    """
-    return RedirectResponse(
-        url=url,
-        status_code=status_code,
-        headers=headers,
-        background=background,
-        encoders=encoders,
-    )
